"use strict";

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var config = require('./config');

var databases = config.databases;
var database = {};

database.createDatabase = function (options) {
  var dbName = options.dbName,
      timestampData = options.timestampData,
      autoload = options.autoload,
      done = options.done,
      onload = options.onload,
      afterSerialization = options.afterSerialization,
      beforeDeserialization = options.beforeDeserialization,
      corruptAlertThreshold = options.corruptAlertThreshold,
      compareStrings = options.compareStrings;
  this.name = dbName;
  this.timestampData = timestampData;
  this.autoload = autoload;
  this.done = done;
  this.onload = onload;
  this.afterSerialization = afterSerialization;
  this.beforeDeserialization = beforeDeserialization;
  this.corruptAlertThreshold = corruptAlertThreshold;
  this.compareStrings = compareStrings;
  config.createDatabase.apply(this);
};
/**
 * Select database.
 */


database.select = function (schema) {
  /**
  * @param {string} schema - Schema é a coleção à qual você se refere. Já que a biblioteca não usa referências em variáveis, utilizamos esse meio para referenciar um database e alterar ele sem disponibilizar uma variável global no sistema.
  */
  this.schema = schema;
  return this;
};

database.find = function (query, options, projection) {
  var _this = this;

  var _options = _objectSpread({}, options),
      limit = _options.limit,
      skip = _options.skip,
      sort = _options.sort;

  return new Promise(function (resolve, reject) {
    databases[_this.schema].find(query, projection).sort(sort).limit(limit).skip(skip).exec(function (err, docs) {
      if (err) reject(new Error('Something went wrong', err));
      resolve(docs);
    });
  });
};

database.insert = function (doc) {
  var _this2 = this;

  return new Promise(function (resolve, reject) {
    databases[_this2.schema].insert(doc, function (err, newDoc) {
      if (err) reject(new Error(err));
      resolve(newDoc);
    });
  });
};

database.count = function (query) {
  var _this3 = this;

  return new Promise(function (resolve, reject) {
    databases[_this3.schema].count(query, function (err, count) {
      if (err) reject(new Error(err));
      resolve(count);
    });
  });
};

database.remove = function (query, multi, options) {
  var _this4 = this;

  return new Promise(
  /*#__PURE__*/
  function () {
    var _ref = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee(resolve, reject) {
      var docs;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (!(options && options.returnDeletedData)) {
                _context.next = 4;
                break;
              }

              _context.next = 3;
              return _this4.select(_this4.schema).find(query);

            case 3:
              docs = _context.sent;

            case 4:
              if (!(docs && docs.length > 1 && !multi)) {
                _context.next = 6;
                break;
              }

              throw new Error("You are trying to delete one document, but 'multi' property is set to false");

            case 6:
              databases[_this4.schema].remove(query, {
                multi: multi
              }, function (err, numRemoved) {
                if (err) reject(err);
                resolve({
                  numRemoved: numRemoved,
                  docs: docs
                });
              });

            case 7:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function (_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }());
};

database.flush = function (returnRemoved) {
  var _this5 = this;

  return new Promise(
  /*#__PURE__*/
  function () {
    var _ref2 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee2(resolve, reject) {
      var docs;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              if (!returnRemoved) {
                _context2.next = 4;
                break;
              }

              _context2.next = 3;
              return _this5.select(_this5.schema).find();

            case 3:
              docs = _context2.sent;

            case 4:
              databases[_this5.schema].remove({}, {
                multi: true
              }, function (err, numRemoved) {
                if (err) {
                  reject(new Error(err));
                }

                resolve({
                  numRemoved: numRemoved,
                  docs: docs ? docs : null
                });
              });

            case 5:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    return function (_x3, _x4) {
      return _ref2.apply(this, arguments);
    };
  }());
};

database.update = function (query, updateDoc) {
  var _this6 = this;

  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return new Promise(function (resolve, reject) {
    databases[_this6.schema].update(query, updateDoc, options, function (err, numAffected, affectedDocuments, upsert) {
      if (err) reject(new Error(err));
      resolve({
        numAffected: numAffected,
        affectedDocuments: affectedDocuments,
        upsert: upsert
      });
    });
  });
};

database.drop = function () {
  return new Promise(
  /*#__PURE__*/
  function () {
    var _ref3 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee3(resolve, reject) {
      var DBOpenRequest, deleteRow;
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              deleteRow = function _ref4(db) {
                var transaction = db.transaction(['nedbdata'], 'readwrite');

                transaction.oncomplete = function (event) {
                  console.log('complete');
                };

                transaction.onerror = function () {
                  console.log('complete');
                };

                var ObjectStore = transaction.objectStore('nedbdata');
                var objectStoreRequest = ObjectStore["delete"](this.schema);

                objectStoreRequest.onsuccess = function (event) {
                  console.log('deleted this database');
                };
              };

              DBOpenRequest = window.indexedDB.open("NeDB", 4);

              DBOpenRequest.onsuccess = function (event) {
                var db = DBOpenRequest.result;
                var timeout = setTimeout(function () {
                  deleteRow.call(database, db);
                  clearTimeout(timeout);
                  resolve();
                }, 100);
              };

            case 3:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3);
    }));

    return function (_x5, _x6) {
      return _ref3.apply(this, arguments);
    };
  }());
};

database.index = function (options) {
  return new Promise(function (resolve, reject) {
    if (!options || !options.fieldName) {
      throw new Error('You have to specify a field name to index.');
    }

    db.ensureIndex(options, function (err) {
      if (err) reject(new Error(err));
      resolve("".concat(options.fieldName, " has been indexed"));
    });
  });
};

database._nedb = databases;
module.exports = database;